//
//  StripProtocol.h
//  iStrip
//
//  Created by Joar Wingfors on Sat Feb 01 2003.
//  Copyright (c) 2003 joar.com. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "HTTPConnection.h"
#import "StripOperationProgress.h"

// This is the current version of the protocol
#define STRIP_PLUGIN_PROTOCOL_CURRENT_VERSION 5

// This is the minimum required protocol version
#define STRIP_PLUGIN_PROTOCOL_REQUIRED_VERSION 5

@protocol StripStateDelegate
// The strip will use the StripStateDelegate to report state changes.

- (void) stripDidChangeEnabledState;
- (void) stripDidChangeAvailabilityState;
- (void) stripDidChangeBusyState;

@end

@protocol StripPreloadDelegate
// The strip will use the StripPreloadDelegate to report back on the 
// progress of preloading image data.

- (void) willInitiatePreload;
- (void) preloadDidCancelWithReason:(NSString *) reason;
- (void) preloadDidFailWithError:(StripOperationErrorCode) error description:(NSString *) description;
- (void) didUpdatePreloadProgress:(StripOperationProgress *) progress;
- (void) didCompletePreload;

@end

typedef unsigned int StripValue;

@protocol StripImageDataDownloadDelegate
// The strip will use the StripImageDataDownloadDelegate to report back 
// on the progress of downloading image data.

- (void) imageDownloadForValue:(StripValue) stripValue failedWithError:(StripOperationErrorCode) error description:(NSString *) description;
- (void) imageDownloadForValue:(StripValue) stripValue didUpdateProgress:(StripOperationProgress *) progress;
- (void) imageDownloadForValue:(StripValue) stripValue didDownloadData:(NSData *) data ofType:(NSString *) type;
					  // title:(NSString *) title description:(NSString *) description;

@end

@protocol StripImageDataDownloadValue < NSObject >
// The StripImageDataDownloadValue encapsulates the value currently being
// downloaded.
@end

@protocol StripProtocol < NSObject >

// Should return STRIP_PLUGIN_PROTOCOL_CURRENT_VERSION 
+ (int) stripPluginProtocolVersion;

// Should return the lowest defined protocol version that the
// strip supports. Currently that would be
// STRIP_PLUGIN_PROTOCOL_REQUIRED_VERSION, since it is the only define.
// In the future it could be STRIP_PLUGIN_SOME_PARTICULAR_FEATURE as
// in the example above.
+ (int) stripPluginProtocolVersionRequired;

// Unique identifier for the plugin. This could be the name of the
// principal class, if it isn't to generic.
// This identifier should never be changed, as it is used as a key to
// access preference data.
+ (NSString *) pluginId;

// Human readable name of the plugin. Example: "Comics".
// This name is use to tell strips apart, when strips from different
// plugins have the same name.
// The returned string must not include characters that can not be
// part of a file name (ie. "/" and ":").
+ (NSString *) pluginName;

// This method should return an array of instances of the strip plugin
// class. Usually this would be only one instance, but for sites that
// present multiple comic strips, you would return one instance per
// available strip.
+ (NSArray *) strips;

// Unique identifier for the strip. Example: "com.comics.dilbert".
// This identifier should never be changed, as it is used as a key to
// access preference data.
- (NSString *) stripId;

// Human readable name of the strip. Example: "Dilbert".
// The returned string must not include characters that can not be
// part of a file name (ie. "/" and ":").
- (NSString *) stripName;

- (BOOL) isEnabled;
- (BOOL) isAvailabilityKnown;
- (BOOL) isBusy;
- (BOOL) isRetired;

- (void) setStateDelegate:(id < StripStateDelegate >) delegate;
- (id < StripStateDelegate >) stateDelegate;

- (void) setPreloadDelegate:(id < StripPreloadDelegate >) delegate;
- (id < StripPreloadDelegate >) preloadDelegate;

- (void) setDownloadDelegate:(id < StripImageDataDownloadDelegate >) delegate;
- (id < StripImageDataDownloadDelegate >) downloadDelegate;

- (void) downloadImageDataForStripValue:(StripValue) value;
- (void) cancelDownload;

// Value for first and last strip in archive.
- (StripValue) firstValue;
- (StripValue) lastValue;

// Used to filter out specific values where the strips are missing.
// The value passed to this method will never be outside of the bounds set by the
// "-firstValue" and "-lastValue" methods.
- (BOOL) hasValue:(StripValue) value;

- (StripValue) nextValueSmallerThanValue:(StripValue) value;
- (StripValue) nextValueGreaterThanValue:(StripValue) value;

// Called when the user request to go to the online version of the strip.
// This method should attempt to return an URL for the page in the online archive
// matching the date provided. It should return some sort of fallback URL, even if
// there isn't any page for the exact date provided.
// In contrast to the previous method this method needs to return an URL
// synchronously, and quickly - as it is called on a menu item action.
// The value passed to this method will never be outside of the bounds set by the
// "-firstValue" and "-lastValue" methods.
- (NSURL *) webURLForValue:(StripValue) value;
- (NSURL *) webURL; // Fallback URL

@end

// Informal protocol
@protocol StripOptionalMethodsProtocol

// Return the primary language of the strip in the 2-letter ISO 639 format.
// Example: "en", "se", "jp".
// If this method isn't implemented, English is assumed.
- (NSString *) language;

// Used to provide the image type for the strips that doesn't provide
// that information in the image URL, ie. "http://domain.com/image.php".
- (NSString *) imageType;

// Used to provide a referrer string for sites that require this to
// allow download of images.
- (NSString *) referrer;

// Perform any setup in this method and not in init.
// Don't assume that the plugin is enabled at this point. The strip will receive
// one of the two methods below to indicate the initial enabled / disabled state.
// In particular, do not initiate any network activity if the plugin is not
// enabled.
- (void) setupStripPlugin;

// Notifies the receiver that it has been enabled or disabled by the receiver.
// These methods will only be called when the state is actually changed.
- (void) stripDidDisable;
- (void) stripDidEnable;

// Notifies the receiver that the system is about to go to sleep, or that it
// did awake from sleep. The plugin should cancel any network activity when
// the system goes to sleep.
- (void) systemWillSleep;
- (void) systemDidWake;

// Notifies the receiver that it is time to discard and update availability information.
- (void) updateAvailability;

@end
